#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import pd_series_to_np_array

import numpy as np


@pd_series_to_np_array
def apply_across(*args, fn=None):
    # Apply function across every `i` index for multiple lists

    if len(args) <= 0:
        raise StandardError("Enter more than one argument.")
    elif fn == None:
        raise ValueError("Enter a function name.")

    applied_across = getattr(np, fn)(args, axis=0)

    return applied_across



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype

from .rolling import rolling

import pandas as pd


@args_to_dtype(pd.DataFrame)
def cmf(df, n=2):
    """ Returns the Chaikin Money Flow of a OHLCV dataframe"""

    high = df["high"].values
    low = df["low"].values
    close = df["close"].values
    volume = df["volume"].values

    avg = (2 * close - high - low) / (high - low + 0.000000001) * volume

    avg_roll = rolling(avg, fn="sum", n=n)
    vol_roll = rolling(volume, fn="sum", n=n)

    _chaikin_money_flow = avg_roll / vol_roll
    _chaikin_money_flow[:n] = 0

    return _chaikin_money_flow



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.wrappers import args_to_dtype

import pandas as pd


def group_candles(df, n=4):
    """ Combine candles so instead of needing one dataset for each time interval,
        you can form time intervals using more precise data.

    Example: You have 15-min candlestick data but want to test a strategy based
        on 1-hour candlestick data  (n=4).
    """

    columns = ["date", "open", "high", "low", "close", "volume"]

    try:
        candles = df[columns].values
        results = []
        for i in range(0, len(df) - n, n):
            results.append(
                [
                    candles[i, 0],  # date
                    candles[i, 1],  # open
                    candles[i : i + n, 2].max(),  # high
                    candles[i : i + n, 3].min(),  # low
                    candles[i + n, 4],  # close
                    candles[i : i + n, 5].sum(),  # volume
                ]
            )
        return pd.DataFrame(results, columns=columns)

    # File does not contain all columns needed to group candles
    except KeyError as e:
        raise KeyError(
            f'Column headers not compatable.  You need to have at least \
                        a "date", "open", "high", "low", "close", and "volume" column'
        )



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.group_candles import group_candles
from TAcharts.utils.draw_candlesticks import draw_candlesticks

from .sma import sma

from datetime import datetime, timedelta
import matplotlib.dates as mdates

import matplotlib.pyplot as plt

import pandas as pd


class Ichimoku:
    def __init__(self, df=None, filename=None, interval=None):

        if filename:
            filename_abs_path = f"{os.getcwd()}/{filename}"
            try:
                df = pd.read_csv(filename_abs_path)
            except:
                raise FileNotFoundError(f"{filename_abs_path}\n\nDoes not exist.")

        # Now that we have our df, make all headers lowercase
        df.columns = [col.lower() for col in df]

        if interval:
            df = group_candles(df, interval)

        self.df = df
        self.ichimoku = {}

        self._validate_data(interval)

    def _validate_data(self, interval):
        """ Make sure we have enough data to form the cloud """

        if interval > len(self.df):
            raise AssertionError(
                f"Error: make sure the dataset has more than {interval} rows."
            )

    def _extend_window(self, displacement):
        last_date = self.df["date"].iat[-1]
        window = last_date - self.df["date"].iat[-2]
        new_dates = [last_date + window * (1 + i) for i in range(displacement)]
        self.df = self.df.append(
            pd.DataFrame(new_dates, columns=["date"]), ignore_index=True, sort=False
        )
        return

    def _build_lines(self, **kwargs):
        for key, val in kwargs.items():
            high = sma(self.df["high"], val, "max")
            low = sma(self.df["low"], val, "min")
            self.ichimoku[key] = (high + low) / 2
        return

    def _build_spans(self, senkou_b_interval, displacement):
        self.ichimoku["chikou"] = self.df["close"].shift(-displacement)
        self.ichimoku["senkou_a"] = (
            self.ichimoku["tenkan"] + self.ichimoku["kijun"]
        ) / 2

        self._build_lines(senkou_b=senkou_b_interval)

        self.ichimoku["senkou_a"] = self.ichimoku["senkou_a"].shift(displacement)
        self.ichimoku["senkou_b"] = self.ichimoku["senkou_b"].shift(displacement)

        for indicator in ["tenkan", "kijun", "chikou", "senkou_a", "senkou_b"]:
            self.ichimoku[indicator] = self.ichimoku[indicator][
                senkou_b_interval + displacement :
            ]

        self.df = self.df[senkou_b_interval + displacement :]
        return

    def build(self, tenkan_interval, kijun_interval, senkou_b_interval, displacement):
        """ Create Ichimoku data """

        self._validate_data(senkou_b_interval)
        self._extend_window(displacement)

        # Tenkan (conversion line) = (highest high + highest low)/2 for the past x intervals
        # Kijun (base line) = (highest high + lowest low)/2 for the past x*3 intervals
        self._build_lines(tenkan=tenkan_interval, kijun=kijun_interval)

        # Chikou (lagging span) = Current closing price time-shifted backwards x*3 intervals
        # Senkou span A (leading span A) = (tenkan + kijun)/2 time-shifted forwards x*3 intervals
        # Senkou span B (leading span B) = (highest high + lowest low)/2 for past x*6 intervals, shifted forwards x*3 intervals
        self._build_spans(
            senkou_b_interval=senkou_b_interval, displacement=displacement
        )

        return self.ichimoku

    def plot(self):
        fig, ax = plt.subplots(figsize=(30, 10))

        x = self.df["date"]
        try:
            x = pd.to_datetime(x)
        except:
            x = range(len(self.df))

        plt.plot(x, self.ichimoku["tenkan"], color="blue")
        plt.plot(x, self.ichimoku["kijun"], color="maroon")
        plt.plot(x, self.ichimoku["senkou_a"], color="green", linewidth=0.5)
        plt.plot(x, self.ichimoku["senkou_b"], color="red", linewidth=0.5)

        draw_candlesticks(ax, self.df)

        fig.suptitle("Ichimoku", fontsize=30)
        plt.ylabel("BTC price ($)")
        plt.rc("axes", labelsize=20)
        plt.rc("font", size=18)

        ax.xaxis.set_major_locator(mdates.MonthLocator())
        ax.xaxis.set_major_formatter(mdates.DateFormatter("%m/%d/%y"))
        plt.xticks(rotation=45)

        plt.fill_between(
            x,
            self.ichimoku["senkou_a"],
            self.ichimoku["senkou_b"],
            where=self.ichimoku["senkou_a"] >= self.ichimoku["senkou_b"],
            facecolor="limegreen",
            interpolate=True,
        )
        plt.fill_between(
            x,
            self.ichimoku["senkou_a"],
            self.ichimoku["senkou_b"],
            where=self.ichimoku["senkou_a"] <= self.ichimoku["senkou_b"],
            facecolor="salmon",
            interpolate=True,
        )
        return plt.show()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-

import os.path
import codecs
import setuptools

# Get the long description from the relevant file
here = os.path.abspath(os.path.dirname(__file__))
with codecs.open(os.path.join(here, 'README.md'), encoding='utf-8') as readme:
    long_description = readme.read()


setuptools.setup(

  name='TAcharts',

  version='0.0.24',

  author='Carter Carlson',

  author_email='carlfarterson@gmail.com',

  license='MIT',

  description='TA Charting tool',

  keywords=['TA', 'mathematics', 'algorithms'],

  long_description=long_description,

  long_description_content_type='text/markdown',

  url='https://github.com/carlfarterson/TAcharts',

  packages=setuptools.find_packages(),

  install_requires=[
    'pandas>=0.21.0',
    'numpy>=1.0.0',
    'matplotlib>=2.0.0'
  ],

  classifiers=[
    'Development Status :: 3 - Alpha',
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: MIT License',
    'Intended Audience :: Developers'
  ],

  python_requires='>=3.6'

)



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.group_candles import group_candles
from TAcharts.utils.draw_candlesticks import draw_candlesticks

import matplotlib.dates as mdates
import matplotlib.pyplot as plt

from .sma import sma
from .sdev import sdev

import pandas as pd
import os


class Bollinger:
    def __init__(self, df=None, filename=None, interval=None):
        if filename:
            filename_abs_path = f"{os.getcwd()}/{filename}"
            try:
                df = pd.read_csv(filename_abs_path)
            except:
                raise FileNotFoundError(f"{filename_abs_path}\n\nDoes not exist.")

        # Make all columns lowercase
        df.columns = [col.lower() for col in df]

        if interval:
            df = group_candles(df, interval)

        self.df = df
        self.bollinger = {}

    def _validate_data(self, interval):
        """ Make sure we have enough data """

        if interval > len(self.df):
            raise AssertionError(
                f"Error: make sure the dataset has more than {interval} rows."
            )

    def build(self, n=20, ndev=2):
        """ Create Bollinger data """

        self._validate_data(n)

        # Create base simple moving average
        self.bollinger["sma"] = sma(self.df["close"], n=n)

        rng = ndev * sdev(self.df["close"], n=n)

        # Create upper and lower bands
        self.bollinger["h_band"] = self.bollinger["sma"] + rng
        self.bollinger["l_band"] = self.bollinger["sma"] - rng

        self.n = n
        self.ndev = ndev
        return self.bollinger

    def plot(self):
        fig, ax = plt.subplots(1, figsize=(20, 10))

        try:
            x = pd.to_datetime(self.df["date"])
        except:
            x = range(len(self.df))

        plt.plot(x, self.bollinger["l_band"], color="blue", label=None)
        plt.plot(x, self.bollinger["h_band"], color="blue", label=None)
        plt.plot(
            x, self.bollinger["sma"], color="orange", label="Simple Moving Average"
        )

        draw_candlesticks(ax, self.df)

        plt.rc("axes", labelsize=20)
        plt.rc("font", size=18)

        ax.xaxis.set_major_formatter(mdates.DateFormatter("%m/%d/%y"))
        ax.xaxis.set_major_locator(mdates.MonthLocator())
        plt.xticks(rotation=45)

        fig.suptitle("Bollinger Bands", fontsize=30)
        plt.ylabel("BTC price ($)")
        plt.legend(f"{self.n}MA")

        return plt.show()



#!/usr/bin/env python
# -*- coding: utf-8; py-indent-offset:4 -*-
from __future__ import absolute_import, division, print_function, unicode_literals

from TAcharts.utils.demo_df import demo_df

from .atr import atr

import matplotlib.pyplot as plt
import matplotlib.patches as patches

import numpy as np
import pandas as pd
import os


class Renko:
    def __init__(self, df=None, filename=None, interval=None):
        if filename:
            filename_abs_path = f"{os.getcwd()}/{filename}"
            try:
                df = pd.read_csv(filename_abs_path)
            except:
                raise FileNotFoundError(f"{filename_abs_path}\n\nDoes not exist.")
        elif df is None:
            try:
                df = demo_df()
            except:
                return f"Figure out a dataset or connect to the internet, ya goof"

        # Group candles if necessary
        if interval:
            df = group_candles(df, interval)

        # Shift date once to account for close
        df["date"] = df["date"].shift(1)

        self.df = df

    def set_brick_size(self, brick_size=None, auto=True, atr_interval=14):
        """ Setting brick size """

        if len(self.df) < atr_interval:
            raise ValueError("ATR interval is longer than historical data.")

        self.brick_size = self._optimize_brick_size(auto, brick_size, atr_interval)
        return self.brick_size

    def _optimize_brick_size(self, auto, brick_size, atr_interval):
        """ Helper function to get optimal brick size based on ATR """

        if auto and not brick_size:
            average_true_range = atr(
                self.df["high"], self.df["low"], self.df["close"], n=atr_interval
            )
            brick_size = np.median(average_true_range)
        return brick_size

    def _apply_renko(self, i):
        """ Determine if there are any new bricks to paint with current price """

        num_bricks = 0

        gap = (self.close[i] - self.renko["price"][-1]) // self.brick_size
        direction = np.sign(gap)
        # No gap means there's not a new brick
        if direction == 0:
            return
        # Add brick(s) in the same direction
        if (gap > 0 and self.renko["direction"][-1] >= 0) or (
            gap < 0 and self.renko["direction"][-1] <= 0
        ):
            num_bricks = gap
        # Gap >= 2 or -2 and opposite renko direction means we're switching brick direction
        elif np.abs(gap) >= 2:
            num_bricks = gap - 2 * direction
            self._update_renko(i, direction, 2)

        for brick in range(abs(int(num_bricks))):
            self._update_renko(i, direction)

        return

    def _update_renko(self, i, direction, brick_multiplier=1):
        """ Append price and new block to renko dict """

        renko_price = self.renko["price"][-1] + (
            direction * brick_multiplier * self.brick_size
        )
        self.renko["index"].append(i)
        self.renko["price"].append(renko_price)
        self.renko["direction"].append(direction)
        self.renko["date"].append(self.df["date"].iat[i])
        return

    def build(self):
        """ Create Renko data """

        units = self.df["close"].iat[0] // self.brick_size
        start_price = units * self.brick_size

        # Create Genesis block
        self.renko = {
            "index": [0],
            "date": [self.df["date"].iat[0]],
            "price": [start_price],
            "direction": [0],
        }

        # Run price feed through our renko model
        for i in range(1, len(self.close)):
            self._apply_renko(i)

        return self.renko

    def plot(self, num_bricks=None, signal_indices=None):

        prices = self.renko["price"]
        directions = self.renko["direction"]
        brick_size = self.renko["brick_size"]

        # Limits display to a custom sized set
        if num_bricks:
            prices = prices[-num_bricks:]
            directions = directions[-num_bricks:]

        if signal_indices:
            for index in signal_indices:
                plt.axvline(x=index)

        # Create `fig` and `ax`
        fig, ax = plt.subplots(1, figsize=(10, 5))

        # Add title to chart
        fig.suptitle(f"Renko Chart (brick size = {round(brick_size, 2)})", fontsize=20)

        # Add label to Y-axis
        ax.set_ylabel("Price ($)")

        plt.rc("axes", labelsize=20)
        plt.rc("font", size=16)

        # Setup X-axis
        x_min = 0
        x_max = len(prices) + 1
        ax.set_xlim(x_min, x_max)

        # Setup Y-axis
        y_min = min(prices) - 2 * brick_size
        y_max = max(prices) + 2 * brick_size
        ax.set_ylim(y_min, y_max)

        # Add bricks to chart
        for x, (price, direction) in enumerate(zip(prices, directions)):

            # Setting brick color and y-position
            if direction == 1:
                facecolor = "g"
                y = price - brick_size
            else:
                facecolor = "r"
                y = price

            ax.add_patch(
                patches.Rectangle(
                    (x + 1, y),
                    height=brick_size,
                    width=1,
                    facecolor=facecolor,  # Either Green or Red
                )  # end of patches.Rectangle
            )  # end of ax.add_patch

        return plt.show()
